capture log close
clear all
macro drop _all
version 17.0
cd "C:\Users\alice\OneDrive\Alice QUANTONOMICS\Stata\DNSP Option5"
set more off
log using anOpexReg_op5-full, replace text
set maxiter 200
/*--------------------------------------------------------------------------
Project: Quantonomics DNSP Opex Cost Function
Author:  Michael Cunningham (based on a Denis Lawrence program)
         with modifications by Alice Giovani
File:    anOpexReg_op5.do
Initial Date:    4-Jul-2022
Revised Date:	 10-Aug-2023

Purpose: Real opex cost function using 2 specification (CD and TL) and 
	using 2 stochastic structures: SFA and LSE with panel-corrected SEs
Note: Change 'startyr' (at line 25) to 2006 for full sample results and to
	2012 for the latter-half period. Change 'endyr' (at line 26) to 2022 
	to produce results comparative to the previous study.
--------------------------------------------------------------------------*/
tempfile data data1

* Preliminaries

local startyr 2006		// for Aust DNSPs
local endyr   2022

scalar define ppp_aust   		   = 1.527404363
scalar define ppp_nz    		   = 1.482877481
scalar define ppp_canada		   = 1.249076963

scalar define propex_rebase_aust   = 1.238501756
scalar define propex_rebase_nz     = 1.219668445
scalar define propex_rebase_canada = 1.160126855  

* Read data

use DNSPopex23_op5, clear
xtset eiid year

gen int dnsp = eiid - 1000 if country == 1
qui tabulate dnsp, gen(d)
forvalues k = 1/13 {
	replace d`k' = 0 if d`k' == .
}

tabulate country, gen(jur)
correlate custnum circlen rmdem shareugc
tabulate year

summarize d* jur*

* Sample selection
*-------------------------

drop if country == 3 & year < (`startyr' - 1)
drop if country < 3 & year < `startyr'
drop if country == 3 & year > (`endyr' - 1)
drop if country < 3 & year > `endyr'

* Modify & create variables
*--------------------------

* Adressing change of VIC's DNSP from calendar to financial year in 2021

generate yr=year
replace yr=yr+0.5 if dnsp==3 & year<2021
replace yr=yr+0.5 if dnsp==8 & year<2021
replace yr=yr+0.5 if dnsp==9 & year<2021
replace yr=yr+0.5 if dnsp==11 & year<2021
replace yr=yr+0.5 if dnsp==13 & year<2021
replace yr=yr-0.25 if country==2
replace yr=yr+0.5 if country==3

replace propex = propex / propex_rebase_aust    if country==1
replace propex = propex / propex_rebase_nz      if country==2
replace propex = propex / propex_rebase_canada  if country==3
replace propex = propex * ppp_nz / ppp_aust     if country==2
replace propex = propex * ppp_canada / ppp_aust if country==3

*gen lastob = 0
*replace lastob = 1 if dnsp != dnsp[_n+1]

* Regression log variables

gen lvc = log(opex / propex)
gen ly1 = log(custnum)
gen ly2 = log(circlen)
gen ly3 = log(rmdem)
gen lz1 = log(shareugc)

* mean correct output data
qui reg ly1
predict mly1
replace ly1=ly1-mly1
qui reg ly2
predict mly2
replace ly2=ly2-mly2
qui reg ly3
predict mly3
replace ly3=ly3-mly3

* TLG variables

gen ly11 = (ly1^2)/2
gen ly12 = ly1*ly2
gen ly13 = ly1*ly3
gen ly22 = (ly2^2)/2
gen ly23 = ly2*ly3
gen ly33 = (ly3^2)/2


* Regression models
*---------------------- 

* 1. PCSECD

xtpcse lvc ly1 ly2 ly3 lz1 yr jur2 jur3 d2-d13, c(a) het
estimates store PCSECD

* 2. PCSETLG

xtpcse lvc ly1 ly2 ly3 ly11 ly12 ly13 ly22 ly23 ly33 lz1 yr jur2 jur3 ///
	d2-d13, c(a) het
estimates store PCSETLG
test ly11 ly12 ly13 ly22 ly23 ly33
	
gen el1_pcse = _b[ly1] + _b[ly11]*ly1 + _b[ly12]*ly2 + _b[ly13]*ly3 
gen el2_pcse = _b[ly2] + _b[ly12]*ly1 + _b[ly22]*ly2 + _b[ly23]*ly3 
gen el3_pcse = _b[ly3] + _b[ly13]*ly1 + _b[ly23]*ly2 + _b[ly33]*ly3

display "ELASTICITIES"
tabstat el1_pcse el2_pcse el3_pcse, by(country) statistics(mean)
tabstat el1_pcse el2_pcse el3_pcse if dnsp <= 13, by(dnsp) statistics(mean)

display "MONOTONICITY VIOLATIONS"
gen mon1 = cond(el1_pcse < 0,1,0)	
gen mon2 = cond(el2_pcse < 0,1,0)	
gen mon3 = cond(el3_pcse < 0,1,0)
tabstat mon1 mon2 mon3, by(country) statistics(mean)
tabstat mon1 mon2 mon3 if dnsp <= 13, by(dnsp) statistics(mean) 

* exporting  monotonicity and elasticity data to excel 
save  `data'  
keep eiid year country custnum rmdem circlen  ///
	     el1_pcse el2_pcse el3_pcse mon1 mon2 mon3

export excel eiid year country custnum rmdem circlen ///
		     el1_pcse el2_pcse el3_pcse mon1 mon2 mon3 ///
			 using monotonicity_op5.xlsx, sheet("lsetlg", replace) firstrow(var)
	 
use `data', clear		 

drop mon1 mon2 mon3

* 3. SFACD

qui reg lvc ly1 ly2 ly3 lz1 yr jur2 jur3
matrix b0 = e(b), .1, ln(e(rmse)^2) , .1
xtfrontier lvc ly1 ly2 ly3 lz1 yr jur2 jur3, cost from(b0, copy) ti
estimates store SFACD

predict tecd, te
frontier_teci tecd
generate effcd = 1/tecd
generate effcd_l = 1/tecd_u
generate effcd_u = 1/tecd_l

display "EFFICIENCY SCORES"
*list dnsp year effcd effcd_l effcd_u if lastob == 1 & dnsp <= 13
tabstat effcd effcd_l effcd_u if dnsp <= 13, by(dnsp) statistics(mean)


* 4. SFATLG

*constraint 1 _b[mu:_cons] == 0

qui reg lvc ly1 ly2 ly3 ly11 ly12 ly13 ly22 ly23 ly33 lz1 yr jur2 jur3
matrix b0 = e(b), .1, ln(e(rmse)^2) , .1
xtfrontier lvc ly1 ly2 ly3 ly11 ly12 ly13 ly22 ly23 ly33 lz1 yr jur2 jur3, ///
	cost from(b0, copy) ti
estimates store SFATLG
test ly11 ly12 ly13 ly22 ly23 ly33

predict tetl, te
frontier_teci tetl
generate efftl = 1/tetl
generate efftl_l = 1/tetl_u
generate efftl_u = 1/tetl_l

display "EFFICIENCY SCORES"
*list dnsp year efftl efftl_l efftl_u if lastob == 1 & dnsp <= 13
tabstat efftl efftl_l efftl_u if dnsp <= 13, by(dnsp) statistics(mean)

gen el1_sfa = _b[ly1] + _b[ly11]*ly1 + _b[ly12]*ly2 + _b[ly13]*ly3 
gen el2_sfa = _b[ly2] + _b[ly12]*ly1 + _b[ly22]*ly2 + _b[ly23]*ly3 
gen el3_sfa = _b[ly3] + _b[ly13]*ly1 + _b[ly23]*ly2 + _b[ly33]*ly3

display "ELASTICITIES"
tabstat el1_sfa el2_sfa el3_sfa, by(country) statistics(mean)
tabstat el1_sfa el2_sfa el3_sfa if dnsp <= 13, by(dnsp) statistics(mean)

display "MONOTONICITY VIOLATIONS"
gen mon1 = cond(el1_sfa < 0,1,0)	
gen mon2 = cond(el2_sfa < 0,1,0)	
gen mon3 = cond(el3_sfa < 0,1,0)
tabstat mon1 mon2 mon3, by(country) statistics(mean)
tabstat mon1 mon2 mon3 if dnsp <= 13, by(dnsp) statistics(mean)

* exporting  monotonicity and elasticity data to excel 
save  `data1'  
keep eiid year country custnum rmdem circlen  ///
	     el1_sfa el2_sfa el3_sfa mon1 mon2 mon3

export excel eiid year country custnum rmdem circlen ///
		     el1_sfa el2_sfa el3_sfa mon1 mon2 mon3 ///
			 using monotonicity_op5.xlsx, sheet("sfatlg", replace) firstrow(var)
	 
use `data1', clear 		 


drop mon1 mon2 mon3

* LR test
lrtest SFATLG SFACD


* Tabulating models
*--------------------

label variable ly1  "ln(Custnum)=x1" 
label variable ly2  "ln(CircLen)=x2" 
label variable ly3  "ln(RMDemand)=x3" 
label variable lz1  "ln(ShareUGC)" 
label variable ly11 "x1*x1/2" 
label variable ly12 "x1*x2" 
label variable ly13 "x1*x3" 
label variable ly22 "x2*x2/2" 
label variable ly23 "x2*x3" 
label variable ly33 "x3*x3/2"
label variable yr   "Year" 
label variable jur2 "New Zealand" 
label variable jur3 "Ontario" 
label variable d2   "AGD" 
label variable d3   "CIT" 
label variable d4   "END" 
label variable d5   "ENX" 
label variable d6   "ERG"
label variable d7   "ESS" 
label variable d8   "JEN" 
label variable d9   "PCR" 
label variable d10  "SAP"
label variable d11  "AND" 
label variable d12  "TND" 
label variable d13  "UED"


outreg2 [PCSECD] using LSECD_op5.xls, sideway stats(coef se tstat) ///
		noparen dec(3) lab excel noaster ctitle("Coefficient"; ///
		"Standard error"; "t-ratio") nodepvar groupvar(ly1 ly2 ly3 lz1 ///
		yr Country jur2 jur3 DNSP d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13) ///
		replace 

outreg2 [PCSETLG] using LSETLG_op5.xls, sideway stats(coef se tstat) ///
		noparen dec(3) lab excel noaster ctitle("Coefficient"; ///
		"Standard error"; "t-ratio") nodepvar ///
		groupvar(ly1 ly2 ly3 ly11 ly12 ly13 ly22 ly23 ly33 lz1 yr Country ///
		jur2 jur3 DNSP d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13) replace 

outreg2 [SFACD] using SFACD_op5.xls, sideway stats(coef se tstat) ///
		noparen dec(3) lab excel noaster ctitle("Coefficient"; ///
		"Standard error"; "t-ratio") nodepvar drop(lnsigma2 lgtgamma) ///
		groupvar(ly1 ly2 ly3 lz1 yr Country jur2 jur3) replace 

outreg2 [SFATLG] using SFATLG_op5.xls, sideway stats(coef se tstat) ///
		noparen dec(3) lab excel noaster ctitle("Coefficient"; ///
		"Standard error"; "t-ratio") nodepvar drop(lnsigma2 lgtgamma) ///
		groupvar(ly1 ly2 ly3 ly11 ly12 ly13 ly22 ly23 ly33 lz1 yr Country ///
		jur2 jur3) replace




log close
exit
